import math
import re
import pandas as pd
from FeatureSelection.select_point_feature import apply_threshold
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')


TEMPLATE_FILE_EVOO = CONFIG['features']['TEMPLATE_FILE_EVOO']
TEMPLATE_FILE_VOO = CONFIG['features']['TEMPLATE_FILE_VOO']
TEMPLATE_FILE_LOO = CONFIG['features']['TEMPLATE_FILE_LOO']
MASK_FILE_EVOO = CONFIG['features']['MASK_FILE_EVOO']
MASK_FILE_VOO = CONFIG['features']['MASK_FILE_VOO']
MASK_FILE_LOO = CONFIG['features']['MASK_FILE_LOO']
DIFF_THRESHOLD = float(CONFIG['features']['DIFF_THRESHOLD'])


def mean(df_values, column):
    sum_value = 0
    count = 0
    
    if column == 'ALL':
        start = df_values.columns.get_loc(CONFIG['columns']['v_start'])  # get index of column v_0
        for index, row in df_values.iterrows(): 
            for a in row[start:]:  # iterate values in row from column v_0 to the end of row (column v_4xx something)
                if not math.isnan(a):
                    sum_value += a
                    count += 1
        if count == 0:
            mean_value = math.nan
        else:
            mean_value = sum_value/count
                    
    else:
        if column not in df_values.columns:
            return None
        mean_value = df_values[column].mean()
                
    return mean_value


def max(df_values, column):
    if column not in df_values.columns:
        return None
    max_value = df_values[column].max()
    return max_value


def min(df_values, column):
    if column not in df_values.columns:
        return None
    min_value = df_values[column].min()
    return min_value


def median(df_values, column):
    if column not in df_values.columns:
        return None
    median_value = df_values[column].median()
    return median_value


def std(df_values, column):
    if column not in df_values.columns:
        return None
    std_value = df_values[column].std(ddof=0)
    return std_value


def delta(df_values, column):
    if column not in df_values.columns:
        return None
    values = df_values[column]
    max_value = values.max()
    min_value = values.min()
    return max_value - min_value


def sum(df_values, column):
    if column not in df_values.columns:
        return None
    sum_value = df_values[column].sum()
    return sum_value


def point(df, attribute):
    test_format = re.search("^.*_[0-9]+-[0-9]+-[0-9]+-[0-9]+$", attribute)
    if not test_format:
        print("Error: wrong attributes")
        return None
    function, location = attribute.split("_")
    value, value_len, spectrum, spectrum_len = location.split("-")

    if 'v_'+value not in df.columns:
        print("Error: column not found")
        return None
    start_value = df.columns.get_loc('v_'+value)
    end_value = start_value + int(value_len) + 1    # +1 because entry 1:2 does not select column 2
    start_spectrum = int(spectrum) - 1              # spectrum starts from 1, not zero
    end_spectrum = start_spectrum + int(spectrum_len) + 1

    if start_value < 0 or end_value > len(df.columns) or start_spectrum < 0 or end_spectrum > len(df.index):
        print("Error: attributes out of range ")
        return None
    selected_part = df.iloc[start_spectrum:end_spectrum, start_value:end_value]

    if function == "max":
        return selected_part.values.max()
    elif function == "mean":
        return selected_part.values.mean()
    else:
        print("Error: function not supported")
        return None


def diff_map(df1, df2, start_column, end_column):
    # check if columns exist
    if start_column not in df1.columns:
        print("Error: column " + start_column + " not found")
        return None
    if end_column not in df1.columns:
        print("Error: column " + end_column + " not found")
        return None
    if start_column not in df2.columns:
        print("Error: column " + start_column + " not found")
        return None
    if end_column not in df2.columns:
        print("Error: column " + end_column + " not found")
        return None

    # set start and end column as an number in all columns
    start_value1 = df1.columns.get_loc(start_column)
    end_value1 = df1.columns.get_loc(end_column) + 1
    start_value2 = df2.columns.get_loc(start_column)
    end_value2 = df2.columns.get_loc(end_column) + 1

    # check if DataFrames have same number of columns
    size1 = end_value1 - start_value1
    size2 = end_value2 - start_value2
    if size1 != size2:
        print("Error: diffrent number of columns")
        return None

    # column in wrong order
    if size1 <= 0:
        c = start_value1 + 1
        start_value1 = end_value1 - 1
        end_value1 = c
    if size2 <= 0:
        c = start_value2 + 1
        start_value2 = end_value2 - 1
        end_value2 = c

    # select only columns with values
    df1 = df1[df1.columns[start_value1:end_value1]].reset_index()
    df2 = df2[df2.columns[start_value2:end_value2]].reset_index()

    result = df1 - df2
    result = result.drop('index', axis='columns')
    return result


def diff(df, oil_type):
    use_mask = False
    if "-" in oil_type:
        splited = oil_type.split("-", 1)
        oil_type = splited[0]
        use_mask = True

    # load template and mask
    switcher_template = {
        "EVOO": CONFIG['features']['TEMPLATE_FILE_EVOO'],
        "VOO": CONFIG['features']['TEMPLATE_FILE_VOO'],
        "LOO": CONFIG['features']['TEMPLATE_FILE_LOO']
    }
    file_name = switcher_template.get(oil_type)
    template = pd.read_csv(file_name)
    if use_mask:
        switcher_mask = {
            "EVOO": CONFIG['features']['MASK_FILE_EVOO'],
            "VOO": CONFIG['features']['MASK_FILE_VOO'],
            "LOO": CONFIG['features']['MASK_FILE_LOO']
        }
        file_name = switcher_mask.get(oil_type)
        mask = pd.read_csv(file_name)

    map_diff = diff_map(df, template, CONFIG['columns']['v_start'], CONFIG['columns']['v_end'])

    # drop nan rows caused by uneven number of spectrums
    map_diff = map_diff.dropna()

    # make positive values from negative numbers
    map_diff = map_diff.abs()

    map_diff = apply_threshold(map_diff, float(CONFIG['features']['DIFF_THRESHOLD']))

    # use mask to filter anomalies within class
    if use_mask:
        map_diff *= mask

    return map_diff.values.sum()
