import os
from sklearn.model_selection import GridSearchCV
from joblib import dump, load
import numpy as np
import pandas as pd
from sklearn import metrics
from sklearn.model_selection import train_test_split
from sklearn.neural_network import MLPClassifier


def train_mlp(train_features, train_labels, tuning=False):
    if tuning:
        best_params = mlp_tuning(train_features, train_labels)
        clf = MLPClassifier()
        clf.set_params(**best_params)
    else:
        selected_params = {'activation': 'relu', 'alpha': 0.0001, 'batch_size': 'auto', 'beta_1': 0.9,
                           'beta_2': 0.999, 'early_stopping': False, 'epsilon': 1e-08, 'hidden_layer_sizes': (100,),
                           'learning_rate': 'constant', 'learning_rate_init': 0.001, 'max_fun': 15000, 'max_iter': 200,
                           'momentum': 0.9, 'n_iter_no_change': 10, 'nesterovs_momentum': True, 'power_t': 0.5,
                           'random_state': None, 'shuffle': True, 'solver': 'adam', 'tol': 0.0001, 'validation_fraction': 0.1,
                           'verbose': False, 'warm_start': False
                           }
        clf = MLPClassifier()
        clf.set_params(**selected_params)
    return clf


def mlp_tuning(train_features, train_labels):
    tuned_parameters = {
        'hidden_layer_sizes': [(50, 50, 50), (50, 100, 50), (100,)],
        'activation': ['tanh', 'relu'],
        'solver': ['sgd', 'adam'],
        'alpha': [0.0001, 0.05],
        'learning_rate': ['constant', 'adaptive'],
    }

    clf = GridSearchCV(MLPClassifier(), tuned_parameters, cv=5, verbose=1, scoring='accuracy')
    clf.fit(train_features, train_labels)

    print("Best parameters set found on development set:")
    print(clf.best_params_)
    print(clf.best_score_)
    return clf.best_params_


def classify_mlp(df):
    labels = np.array(df['class'])
    df = df.drop('class', axis=1)
    df = df.drop('oil_id', axis=1)
    features_array = np.array(df)

    train_features, test_features, train_labels, test_labels = train_test_split(features_array, labels, test_size=0.20)

    if not os.path.exists('mlp.joblib'):
        # training
        clf = MLPClassifier(hidden_layer_sizes=(128, 64, 32), solver='lbfgs', alpha=1e-05, random_state=1, max_iter=300)

    else:
        clf = load('mlp.joblib')

    clf.fit(train_features, train_labels)
    pred_labels = clf.predict(test_features)

    dump(clf, 'mlp.joblib')

    # evaluation
    macro = metrics.f1_score(test_labels, pred_labels, average='macro')
    micro = metrics.f1_score(test_labels, pred_labels, average='micro')
    accuracy = metrics.accuracy_score(test_labels, pred_labels)

    print('accuracy', accuracy)
    print('macro', macro)
    print('micro', micro)

    print(metrics.confusion_matrix(test_labels, pred_labels))


if __name__ == '__main__':
    df = pd.read_csv('features1.csv')
    classify_mlp(df)
