import argparse
import torch
import torch.nn as nn
import torch.nn.functional as F
from torch.utils.data import DataLoader
from sklearn.metrics import accuracy_score
from sklearn.metrics import confusion_matrix
from Neurons.Vgg16Model import Vgg16
from Neurons.ImageDataset import ImageDataset
import configparser

CONFIG = configparser.ConfigParser()
CONFIG.read('../config.ini')

TRAIN_DATASET_PATH = CONFIG['vgg']['train_directory']
TEST_DATASET_PATH = CONFIG['vgg']['test_directory']


def simplify_array(array):
    new_array = []
    for level1_value in array:
        for level2_value in level1_value:
            new_array.append(level2_value)
    return new_array


def classify_vgg16():
    device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')
    model = Vgg16(num_classes=opts.classes).to(device)

    dataloader = DataLoader(ImageDataset(CONFIG['vgg']['train_directory']), shuffle=True, batch_size=16)
    test_dataloader = DataLoader(ImageDataset(CONFIG['vgg']['test_directory']), shuffle=False, batch_size=16)

    learning_rate = opts.l_rate
    optimizer = torch.optim.Adam(model.parameters(), lr=learning_rate, betas=(0.9, 0.999))
    criterion = nn.CrossEntropyLoss()
    num_epochs = opts.numEpochs

    test_acc = []
    for epoch in range(num_epochs):
        print('epocha: ', epoch)
        # Training
        model.train()
        loss_sum = 0
        count = 0
        for (x, y) in dataloader:
            x, y = x.to(device, dtype=torch.float), y.to(device, dtype=torch.long)

            prediction = model(x)
            loss = criterion(prediction, y)
            loss_sum += loss
            count += len(y)

            optimizer.zero_grad()
            loss.backward()

            optimizer.step()
        print('loss: ', loss_sum/count)

        with torch.no_grad():
            predictions = []
            ys = []

            model.eval()
            for test_x, test_y in test_dataloader:
                test_x, test_y = test_x.to(device, dtype=torch.float), test_y.to(device, dtype=torch.long)
                prediction = model(test_x)

                prediction = F.softmax(prediction, dim=1)

                pred_classes = prediction.argmax(axis=1)
                predictions.append(pred_classes.cpu().numpy())
                ys.append(test_y.numpy())

            ys = simplify_array(ys)
            predictions = simplify_array(predictions)

            acc = accuracy_score(ys, predictions)

            test_acc += [acc]
            conf_matrix = confusion_matrix(ys, predictions)
            print('acc: ', acc, '\n', conf_matrix, '\n--------')
    print(test_acc)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--classes', type=int, default=3, help='Number of oil classes (EVOO, VOO, LOO)')
    parser.add_argument('--numEpochs', type=int, default=10, help='Number of epochs')
    parser.add_argument('--l_rate', type=float, default=1e-4, help='Learning rate')
    opts = parser.parse_args()

    classify_vgg16()
